/**
 * sipML5の操作クラス
 */
const script_src = document.currentScript.src;

class SimpleCall {
  constructor(options) {
    const sounds = Object.assign({}, {url: new URL('../sounds/', script_src)}, options?.sounds);
    const audio = this.initAudioTags({
      audio_remote: {autoplay: true},
      dialtone: {loop: true, src: (new URL('dialtone.mp3', sounds.url)).href},
      ringbacktone: {loop: true, src: (new URL('ringbacktone.mp3', sounds.url)).href},
      busytone: {loop: true, src: (new URL('busytone.mp3', sounds.url)).href},
    });
    const defaults = {
      realm: 'ct-tech.biz',
      websocket_url: 'wss://websig1.ct-tech.biz/ws',
      ice_servers: [
        {url: 'stun:turn.ct-tech.biz:443'},
        {url: 'turn:turn.ct-tech.biz:443', username: 'webrtc', credential: 'mypassword'}
      ],
      session_config: {
        audio_remote: audio.audio_remote,
      },
      sounds: {
        dialtone: audio.dialtone,
        ringbacktone: audio.ringbacktone,
        busytone: audio.busytone,
      },
    };
    this.options = Object.assign({}, defaults, options);
  }

  /**
   * HTML:audioタグの初期化
   */
  initAudioTags(audioItems) {
    const result = {};
    Object.keys(audioItems).forEach(item => {
      result[item] = document.getElementById(item);
      if (!result[item]) {
        console.log('Create audio: ' + item);
        result[item] = document.createElement('audio');
        result[item].id = item;
        Object.assign(result[item], audioItems[item]);
        document.body.appendChild(result[item]);
      }
    });
    return result;
  }

  /**
   * 古いiOS Safari対策
   */
  prepareSounds() {
    Object.values(this.options.sounds).forEach(audio => {
      audio.play();
      audio.pause();
    });
  }

  /**
   * 認証情報の取得
   * 
   * @param {*} request HTTPリクエスト情報
   */
  getCredential(request) {
    return fetch(request)
    .then((response) => response.json())
    .then((json) => {
      this.setCredential(json.username, json.password);
    });  
  }

  /**
   * 認証情報の設定
   * 
   * @param {string} username 内線番号
   * @param {string} password パスワード
   */
  setCredential(username, password) {
    if (username) {
      this.options.username = username;
    }
    if (password) {
      this.options.password = password;
    }
  }

  /**
   * 表示名の設定
   * 
   * @param {string} display_name 表示名
   */
  setDisplayName(display_name) {
    this.options.display_name = display_name;
  }

  /**
   * SIP処理の開始
   * 
   * @param {string} username 内線番号
   * @param {string} password パスワード
   */
  start(username, password) {
    if (this.webrtc) {
      return Promise.resolve();
    }
    return new Promise((resolve, reject) => {
      this.setCredential(username, password);
      this.options.delegate = {
        onRegistered: (e) => {
          if (e.type == 'connected') {
            resolve();
          } else {
            this.stop();
            reject('ユーザー認証に失敗しました');
          }
        },
        onStartFailed: (e) => {
          this.stop();
          reject('WebRTCサーバとの通信に失敗しました');
        },
        onIncomingCall: (e) => {
          e.newSession.hangup();
        },
        onTerminatedCall: (e) => {
					this.stop();
          if (this.callback) {
            this.callback();
          }
				},
      };
      this.webrtc = new WebRTC(this.options);  
    });
  }

  /**
   * SIP処理の終了
   */
  stop() {
    if (this.webrtc) {
      this.webrtc.stop();
      this.webrtc = null;
    }
    if (this.callback) {
      this.callback();
    }
  }

  /**
   * 通話処理中か確認
   */
  hasCall() {
    if (this.webrtc) {
      return this.webrtc.hasCall();
    }
    return false;
  }

  /**
   * 電話の発信
   * 
   * @param {string} telnum 発信先電話番号
   * @param {object} callback 終話時コールバック
   */
  call(telnum, callback) {
    // 古いiOS Safari対策
    this.prepareSounds();

    return new Promise((resolve, reject) => {
      this.callback = callback;
      this.start()
      .then(() => {
        if (this.webrtc) {
          if (this.webrtc.call(telnum, 'call-audio')) {
            resolve();
          } else {
            this.stop();
            reject('電話の発信に失敗しました');
          }
        } else {
          this.stop();
          reject('WebRTCが初期化されておりません');
        }
      })
      .catch((error) => reject(error));
    });
  }

  /**
   * 通話の終了
   */
  hangup() {
    if (this.webrtc) {
      this.webrtc.hangup();
    }
  }

  /**
   * フック処理(call/hangup)
   * 
   * @param {string} telnum 発信先電話番号
   * @param {object} callback 各種コールバック
   */
  execute(telnum, callback) {
		if (!this.hasCall()) {
      if (callback.onStart) {
        callback.onStart();
      }
			// 発信処理
			return call.call(telnum, callback.onStop);
		} else {
			// 終話処理
			this.hangup();
      return Promise.resolve();
		}
  }
}
