/**
 * WebRTCの処理クラス(SIPml5を直接操作しなくても良いように)
 */
class WebRTC {
  constructor(options) {
    const defaults = {
      debug_level: 'error',
      register_config: {
        expires: 200,
      },
      session_config: {
        video_local: undefined,
        video_remote: undefined,
        audio_remote: undefined,
      },
      sounds: {
        dialtone: undefined,
        ringtone: undefined,
        ringbacktone: undefined,
        busytone: undefined,
      },
      delegate: {
        onIncomingCall: undefined,
        onEstablishedCall: undefined,
        onTerminatedCall: undefined,
        onRegistered: undefined,
        onStartFailed: undefined,
      },
    };
    this.options = Object.assign({}, defaults, options);

    if (!SIPml.isInitialized()) {
      SIPml.init(() => {
        SIPml.setDebugLevel(this.options.debug_level);
        this.start();
      });
    } else {
      this.start();
    }
  }

  configure(options) {
    this.options = Object.assign({}, this.options, options);
  }

  start() {
    console.log('WebRTC::start', this.options);
    if (!this.stack) {
      this.stack = new SIPml.Stack({
        realm: this.options.realm,
        impi: this.options.username,
        impu: 'sip:' + this.options.username + '@' + this.options.realm,
        password: this.options.password,
        display_name: this.options.display_name ?? this.options.username,
        websocket_proxy_url: this.options.websocket_url,
        ice_servers: this.options.ice_servers,
      });
      this.stack.addEventListener('*', (e) => { console.log(e); });
      this.stack.addEventListener('failed_to_start', (e) => {
        if (this.options.delegate.onStartFailed) {
          this.options.delegate.onStartFailed(e);
        }
        this.stop();
      });
      this.stack.addEventListener('stopped', (e) => { this.stack = null; });
      this.stack.addEventListener('started', (e) => { this.register(); });
      this.stack.addEventListener('i_new_call', (e) => { this.onIncomingCall(e); });

      if (this.stack.start() != 0) {
        console.log('Failed to start the SIP stack');
        this.stack = null;
      }
    }
  }

  stop() {
    console.log('WebRTC::stop');
    if (this.stack) {
      this.stack.removeEventListener('stopped');
      this.stack.removeEventListener('started');
      this.stack.removeEventListener('i_new_call');

      this.stack.stop();
      this.stack = null;
    }
  }

  register() {
    console.log('WebRTC::register(' + this.options.username + ')');
    const session = this.stack.newSession('register', this.options.register_config);
    session.addEventListener('*', (e) => { console.log(e); });
    session.addEventListener('connected', (e) => {
      if (this.options.delegate.onRegistered) {
        this.options.delegate.onRegistered(e);
      }  
    });
    session.addEventListener('terminated', (e) => {
      if (this.options.delegate.onRegistered) {
        this.options.delegate.onRegistered(e);
      }  
    });
    session.register();
  }

  hasCall() {
    return this.session;
  }

  isIncoming() {
    return this.incoming;
  }

  isEstablished() {
    return this.established;
  }

  call(number, type = 'call-audio') {
    if (!SIPml.isReady() || this.hasCall()) {
      console.log('Could not to start the outgoing call');
      return false;
    }
    if (!number || number.length == 0) {
      console.log('Not specified outgoing call number');
      return false;
    }

    console.log('WebRTC::call(' + number + '@' + type + ')');
    this.established = false;
    this.incoming = false;
    this.busytone = true;
    this.session = this.stack.newSession(type, this.options.session_config);
    this.session.addEventListener('*', (e) => { console.log(e); });
    this.session.addEventListener('terminating', (e) => { this.onTerminatedCall(e); });
    this.session.addEventListener('terminated', (e) => { this.onTerminatedCall(e); });
    this.session.addEventListener('connected', (e) => { this.onEstablishedCall(e); });
    this.session.addEventListener('m_early_media', (e) => { this.stopSounds(); });
    this.session.addEventListener('i_ao_request', (e) => {
      const code = e.getSipResponseCode();
      console.log('WebRTC::i_ao_request:' + code);
      if (code == 180 || code == 183) {
        this.playSound(this.options.sounds.ringbacktone);
      }
    });

    if (this.session.call(number) != 0) {
      this.session = null;
      return false;
    }

    this.playSound(this.options.sounds.dialtone);
    return true;
  }

  answer() {
    if (!this.hasCall()) {
      return false;
    }

    console.log('WebRTC::answer');
    this.session.accept(this.options.session_config);
    return true;
  }

  hangup() {
    if (!this.hasCall()) {
      return false;
    }

    console.log('WebRTC::hangup');
    this.busytone = false;
    this.session.hangup();
    return true;
  }

  onIncomingCall(e) {
    if (this.hasCall()) {
      e.newSession.hangup();
      return;
    }

    console.log('WebRTC::onIncomingCall');
    this.established = false;
    this.incoming = true;
    this.session = e.newSession;
    this.session.addEventListener('terminated', (e) => { this.onTerminatedCall(e); });
    this.session.addEventListener('connected', (e) => { this.onEstablishedCall(e); });

    this.playSound(this.options.sounds.ringtone);
    if (this.options.delegate.onIncomingCall) {
      this.options.delegate.onIncomingCall(e);
    }
  }

  onEstablishedCall(e) {
    console.log('WebRTC::onEstablishedCall');
    this.established = true;
    this.busytone = true;
    this.stopSounds();
    if (this.options.delegate.onEstablishedCall) {
      this.options.delegate.onEstablishedCall(e);
    }
  }

  onTerminatedCall(e) {
    console.log('WebRTC::onTerminatedCall');
    this.session = null;
    this.stopSounds();
    if (this.busytone) {
      this.busytone = false;
      this.playSound(this.options.sounds.busytone);
      this.stopTimer = setTimeout(() => {
        this.stopSounds();
      }, 3000);
    }
    if (this.options.delegate.onTerminatedCall) {
      this.options.delegate.onTerminatedCall(e);
    }
  }

  playSound(sound) {
    this.stopSounds();
    if (sound) {
      sound.play();
    }
  }

  stopSounds() {
    if (this.stopTimer) {
      clearTimeout(this.stopTimer);
      this.stopTimer = null;
    }
    if (this.options.sounds.ringbacktone) {
      this.options.sounds.ringbacktone.pause();
    }
    if (this.options.sounds.dialtone) {
      this.options.sounds.dialtone.pause();
    }
    if (this.options.sounds.ringtone) {
      this.options.sounds.ringtone.pause();
    }
    if (this.options.sounds.busytone) {
      this.options.sounds.busytone.pause();
    }
  }
}
